local utils = require("utils")
local vec = require("vector-light")
local mat4 = require('cpml').mat4
local vec3 = require('cpml').vec3

local Cam = {}
Cam.__index = Cam
setmetatable(Cam, {
  __call = function() return setmetatable({}, Cam) end
})

function Cam:getPos(forward)
  if (not forward) or forward == 0 then
    return -self.viewX, -self.viewY, -self.viewZ
  else
    local fx, fy = self:getVec(0, forward)
    return -self.viewX + fx, -self.viewY + fy, -self.viewZ
  end
end

function Cam:setPositionImmediate(x, y, z)
  self.easedX, self.viewX = -x, -x
  self.easedY, self.viewY = -y, -y
  self.easedZ, self.viewZ = -z, -z
end

function Cam:setPosition(x, y, z)
  self.viewX = -x
  self.viewY = -y
  self.viewZ = -z
end


function Cam:spinZ(ddRotZ)
  self.dRotZ = utils.clamp(self.dRotZ + ddRotZ, -1.5, 1.5)
  if math.abs(self.dRotZ) < 0.0005 then
    self.dRotZ = 0 -- otherwise outline flickers
  end
end

-- todo: rename?
function Cam:getFwd(size)
  local kx, ky = vec.rotate(-self.rotZ, 0, size or 1)
  return -self.viewX - kx, -self.viewY - ky
end

function Cam:getFwdVec(size)
  return vec.rotate(-self.rotZ, 0, -(size or 1))
end

function Cam:update(dt)
  self.zoom = self.zoomTarget * 0.3 + self.zoom * 0.7
  self.rotX = self.rotXTarget * 0.3 + self.rotX * 0.7

  self.rotZ = self.rotZ + self.dRotZ
  self.dRotZ = self.dRotZ * 0.8

  self.kx, self.ky = vec.rotate(self.rotZ, 1, 0)

  self.fov = self.targetFov * 0.2 + self.fov * 0.8
  self.easedX = (self.viewX + self.easedX) / 2
  self.easedY = (self.viewY + self.easedY) / 2
  self.easedZ = (self.viewZ + self.easedZ) / 2
end


function Cam:reset()
  self.dRotZ = 0
  self.viewX, self.viewY, self.viewZ = 0, 0, 0
  self.easedX, self.easedY, self.easedZ = 0, 0, 0

  self.zoom = 8.25
  self.zoomTarget = self.zoom

  self.rotX = 5
  self.rotZ = 0

  self.rotXTarget = 5

  self.fov = 60
  self.targetFov = self.fov
  self.proj = mat4.identity()
  self.view = mat4.identity()

  self.pcount = 0
  self.priority = -99999
  self.px, self.py, self.pz = 0, 0, 0
end


function Cam:getMatrices(kind, aspect)
  if kind == "ortho" then
    self.proj = mat4.from_ortho(-10, 10, 10/aspect, -10/aspect, -100, 100)
  else
    self.proj:resetPerspective(self.fov, aspect, 0.01, 1000)
  end

  -- tmp
  self.view:resetTranslation(self.viewX, self.viewY, self.viewZ - 3)
  self.view:rotateZ(self.view, self.rotZ)
  self.view:rotateX(self.view, -self.rotX)
  self.view:translateXYZ(self.view, 0, 0, -self.zoom)

  return self.view, self.proj
end


function Cam:setTarget(x, y, z)
  self.viewX, self.viewY, self.viewZ = -x, -y, -z
end


function Cam:setPriorityTarget(priority, x, y, z)
  if self.lock then return end

  if priority == self.priority then
    self.priority = priority
    self.pcount = self.pcount + 1
    self.px, self.py, self.pz = self.px + x, self.py + y, self.pz + z
  elseif priority > self.priority then
    self.priority = priority
    self.pcount = 1
    self.px, self.py, self.pz = x, y, z
  end
end


function Cam:getVec(addAngle, mag)
  -- addAngle == 0 -> forward
  local fwdX = -(mag or 1) * math.sin(self.rotZ + (addAngle or 0))
  local fwdY = -(mag or 1) * math.cos(self.rotZ + (addAngle or 0))
  return fwdX, fwdY
end

return Cam
