extern Image shadowMap;
extern mat4 shadowView;
extern mat4 shadowProj;

extern int debug = 0;

/* varying vec4 FragPosWorldSpace; */
varying vec4 FragPosDepthSpace;
/* varying vec4 FragPosScreenSpace; */

#ifdef VERTEX
extern mat4 model;
extern mat4 view;
extern mat4 proj;

vec4 position(mat4 transform, vec4 vertex) {
  vec4 frag_pos = model * vertex;

  mat4 depthSpaceMatrix = shadowProj * shadowView;
  mat4 screenSpaceMatrix = proj * view;

  /* FragPosWorldSpace = frag_pos; */
  FragPosDepthSpace = depthSpaceMatrix * frag_pos;
  /* FragPosScreenSpace = screenSpaceMatrix * model; */

  return screenSpaceMatrix * frag_pos;
}
#endif

#ifdef PIXEL
vec4 effect(vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords)
{
  vec4 texel = Texel(tex, VaryingTexCoord.xy); // texture_coords);
  if (texel.a < 0.1) { discard; };

  // vec3 projCoords = FragPosDepthSpace.xyz;
  vec3 projCoords = FragPosDepthSpace.xyz; // / FragPosDepthSpace.w; // perspective divide
  projCoords = projCoords * 0.5 + 0.5;
  float closestDepth = Texel(shadowMap, projCoords.xy).r;
  float currentDepth = 1-projCoords.z; // why `1-` ?

  if (debug == 1) {
    return vec4(closestDepth, 0, 0, 1);
  } else if (debug == 2)
  {
    return vec4(0, currentDepth, 0, 1);
  }

  // shadow
  if (currentDepth > closestDepth) {
    return vec4(0.1, 0.1, 0.1, 1); // shadow
  }

  vec4 texel_shadow = Texel(shadowMap, VaryingTexCoord.xy); // texture_coords);

  return texel * color;
}
#endif
